/*
  OCaml Bindings to the library Fam.

  Copyright (C) 2008  Florent Monnier
  Contact:  <fmonnier@linux-nantes.org>

  This program is free software: you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation, either version 3
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>
*/
#include <fam.h>
#include <caml/mlvalues.h>
#include <caml/memory.h>
#include <caml/alloc.h>
#include <caml/custom.h>
#include <caml/fail.h>
#include <string.h>

void finalize_conn( value v )
{
    FAMConnection *fc;
    fc = (FAMConnection *) Data_custom_val(v);
    if (FAMClose( fc ) < 0) 
    {
        caml_failwith( FamErrlist[FAMErrno] );
    }
    //puts("famConnection finalized / closed");  // Debug
}

static struct custom_operations famconn_custom_ops = {
    identifier: "FAMConnection handling",
    finalize:  finalize_conn,
    compare:     custom_compare_default,
    hash:        custom_hash_default,
    serialize:   custom_serialize_default,
    deserialize: custom_deserialize_default
};

static struct custom_operations famreq_custom_ops = {
    identifier: "FAMRequest handling",
    finalize:    custom_finalize_default,
    compare:     custom_compare_default,
    hash:        custom_hash_default,
    serialize:   custom_serialize_default,
    deserialize: custom_deserialize_default
};

static struct custom_operations famevnt_custom_ops = {
    identifier: "FAMEvent handling",
    finalize:    custom_finalize_default,
    compare:     custom_compare_default,
    hash:        custom_hash_default,
    serialize:   custom_serialize_default,
    deserialize: custom_deserialize_default
};

CAMLprim value
ml_famopen( value unit )
{
    CAMLparam1(unit);
    CAMLlocal1(mlfc);
    FAMConnection fc;

    if(FAMOpen(&fc) < 0)
        caml_failwith("FAMOpen");

    mlfc = caml_alloc_custom( &famconn_custom_ops, sizeof(FAMConnection), 0, 1);
    memcpy( Data_custom_val(mlfc), &fc, sizeof(FAMConnection) );

    CAMLreturn(mlfc);
}

CAMLprim value
ml_famclose( value mlfc )
{
    FAMConnection *fc;
    fc = (FAMConnection *) Data_custom_val(mlfc);
    if (FAMClose( fc ) < 0) 
    {
        caml_failwith( FamErrlist[FAMErrno] );
    }
    return Val_unit;
}

CAMLprim value
ml_fammonitordirectory( value mlfc, value filename )
{
    CAMLparam2(mlfc, filename);
    CAMLlocal1(mlfr);

    FAMConnection *fc;
    FAMRequest fr;
    void* userData = NULL;
    fc = (FAMConnection *) Data_custom_val(mlfc);

    if (FAMMonitorDirectory(fc, String_val(filename), &fr, userData) < 0)
    {
        caml_failwith( FamErrlist[FAMErrno] );
    }

    mlfr = caml_alloc_custom( &famreq_custom_ops, sizeof(FAMRequest), 0, 1);
    memcpy( Data_custom_val(mlfr), &fr, sizeof(FAMRequest) );

    CAMLreturn(mlfr);
}

CAMLprim value
ml_fammonitorfile( value mlfc, value filename )
{
    CAMLparam2(mlfc, filename);
    CAMLlocal1(mlfr);

    FAMConnection *fc;
    FAMRequest fr;
    void* userData = NULL;
    fc = (FAMConnection *) Data_custom_val(mlfc);

    if (FAMMonitorFile(fc, String_val(filename), &fr, userData) < 0)
    {
        caml_failwith( FamErrlist[FAMErrno] );
    }

    mlfr = caml_alloc_custom( &famreq_custom_ops, sizeof(FAMRequest), 0, 1);
    memcpy( Data_custom_val(mlfr), &fr, sizeof(FAMRequest) );

    CAMLreturn(mlfr);
}

CAMLprim value
ml_fampending( value mlfc )
{
    FAMConnection *fc;
    fc = (FAMConnection *) Data_custom_val(mlfc);

    return Val_int( FAMPending(fc) );
}

CAMLprim value
ml_famnextevent( value mlfc )
{
    CAMLparam1(mlfc);
    CAMLlocal1(mlfe);

    FAMEvent fe;
    FAMConnection *fc;
    fc = (FAMConnection *) Data_custom_val(mlfc);

    if (FAMNextEvent( fc, &fe ) < 0)
    {
        caml_failwith( FamErrlist[FAMErrno] );
    }

    mlfe = caml_alloc_custom( &famevnt_custom_ops, sizeof(FAMEvent), 0, 1);
    memcpy( Data_custom_val(mlfe), &fe, sizeof(FAMEvent) );

    CAMLreturn(mlfe);
}

CAMLprim value
famevent_code( value mlfe )
{
    FAMEvent *fe;
    fe = (FAMEvent *) Data_custom_val(mlfe);

    switch (fe->code)
    {
        case FAMChanged:        return Val_int(0); break;
        case FAMDeleted:        return Val_int(1); break;
        case FAMStartExecuting: return Val_int(2); break;
        case FAMStopExecuting:  return Val_int(3); break;
        case FAMCreated:        return Val_int(4); break;
        case FAMMoved:          return Val_int(5); break;
        case FAMAcknowledge:    return Val_int(6); break;
        case FAMExists:         return Val_int(7); break;
        case FAMEndExist:       return Val_int(8); break;
        default:
            caml_failwith("famEvent_code: enum error");
    }
    return Val_unit;
}

CAMLprim value
famevent_filename( value mlfe )
{
    FAMEvent *fe;
    fe = (FAMEvent *) Data_custom_val(mlfe);

    return caml_copy_string( fe->filename );
}

static const enum FAMCodes famCodes_table[] = {
    FAMChanged,
    FAMDeleted,
    FAMStartExecuting,
    FAMStopExecuting,
    FAMCreated,
    FAMMoved,
    FAMAcknowledge,
    FAMExists,
    FAMEndExist,
};

CAMLprim value
famcode_to_string( value ml_famCode )
{
    enum FAMCodes famCode;
    famCode = famCodes_table[Int_val(ml_famCode)];

    switch (famCode) {
        case FAMChanged:        return caml_copy_string("FAMChanged");        break;
        case FAMDeleted:        return caml_copy_string("FAMDeleted");        break;
        case FAMStartExecuting: return caml_copy_string("FAMStartExecuting"); break;
        case FAMStopExecuting:  return caml_copy_string("FAMStopExecuting");  break;
        case FAMCreated:        return caml_copy_string("FAMCreated");        break;
        case FAMMoved:          return caml_copy_string("FAMMoved");          break;
        case FAMAcknowledge:    return caml_copy_string("FAMAcknowledge");    break;
        case FAMExists:         return caml_copy_string("FAMExists");         break;
        case FAMEndExist:       return caml_copy_string("FAMEndExist");       break;
        default:
            caml_failwith("string_of_famCode: enum error");
    }
    return Val_unit;
}

// vim: sw=4 et
