/*
  Basic bindings of the libfam for OCaml.

  Copyright (C) 2008  Florent Monnier
  Contact:  <fmonnier@linux-nantes.org>

  This program is free software: you can redistribute it and/or
  modify it under the terms of the GNU General Public License
  as published by the Free Software Foundation, either version 3
  of the License, or (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program.  If not, see <http://www.gnu.org/licenses/>
*/
#include <fam.h>
#include <caml/mlvalues.h>
#include <caml/fail.h>
#include <caml/alloc.h>

CAMLprim value
ml_famopen( value unit )
{
    FAMConnection* fc;
    fc = malloc(sizeof(FAMConnection));
    if(FAMOpen(fc) < 0)
    {
        caml_failwith("famOpen");
    }
    return (value) fc;
}

CAMLprim value
ml_famopen2( value appName )
{
    FAMConnection* fc;
    fc = malloc(sizeof(FAMConnection));
    if(FAMOpen2(fc, String_val(appName)) < 0)
    {
        caml_failwith("famOpen");
    }
    return (value) fc;
}

CAMLprim value
ml_famclose( value fc )
{
    if (FAMClose( (FAMConnection*) fc ) == -1) 
    {
        caml_failwith(FamErrlist[FAMErrno]);
    }
    free( (FAMConnection*) fc );
    fc = (value) NULL;
    return Val_unit;
}

CAMLprim value
alloc_famrequest( value unit )
{
    FAMRequest *fr;
    fr = malloc(sizeof(FAMRequest));
    return (value) fr;
}

CAMLprim value
free_famrequest( value fr )
{
    free( (FAMRequest*) fr );
    fr = (value) NULL;
    return Val_unit;
}

CAMLprim value
alloc_famevent( value unit )
{
    FAMEvent *fe;
    fe = malloc(sizeof(FAMEvent));
    return (value) fe;
}

CAMLprim value
free_famevent( value fe )
{
    free( (FAMEvent*) fe );
    fe = (value) NULL;
    return Val_unit;
}

CAMLprim value
ml_fammonitordirectory( value fc, value filename, value fr, value userData )
{
    if ((FAMConnection*) fc == NULL) caml_failwith("famMonitorDirectory: wrong famConnection");
    if ((FAMRequest*) fr == NULL) caml_failwith("famMonitorDirectory: wrong famRequest");

    if (FAMMonitorDirectory(
                (FAMConnection*) fc,
                String_val(filename),
                (FAMRequest*) fr,
                (void*) userData) < 0)
    {
        caml_failwith("famMonitorDirectory");
    }
    return Val_unit;
}

CAMLprim value
ml_fammonitorfile( value fc, value filename, value fr, value userData )
{
    if ((FAMConnection*) fc == NULL) caml_failwith("famMonitorFile: wrong famConnection");
    if ((FAMRequest*) fr == NULL) caml_failwith("famMonitorFile: wrong famRequest");

    if (FAMMonitorFile(
                (FAMConnection*) fc,
                String_val(filename),
                (FAMRequest*) fr,
                (void*) userData) < 0)
    {
        caml_failwith("famMonitorFile");
    }
    return Val_unit;
}

CAMLprim value
ml_famsuspendmonitor( value fc, value fr )
{
    if ((FAMConnection*) fc == NULL) caml_failwith("famSuspendMonitor: wrong famConnection");
    if ((FAMRequest*) fr == NULL) caml_failwith("famSuspendMonitor: wrong famRequest");

    if (FAMSuspendMonitor( (FAMConnection*) fc, (FAMRequest*) fr) < 0)
    {
        caml_failwith("famSuspendMonitor");
    }
    return Val_unit;
}

CAMLprim value
ml_famresumemonitor( value fc, value fr )
{
    if ((FAMConnection*) fc == NULL) caml_failwith("famResumeMonitor: wrong famConnection");
    if ((FAMRequest*) fr == NULL) caml_failwith("famResumeMonitor: wrong famRequest");

    if (FAMResumeMonitor( (FAMConnection*) fc, (FAMRequest*) fr ) < 0)
    {
        caml_failwith("famResumeMonitor");
    }
    return Val_unit;
}

CAMLprim value
ml_famcancelmonitor( value fc, value fr )
{
    if ((FAMConnection*) fc == NULL) caml_failwith("famCancelMonitor: wrong famConnection");
    if ((FAMRequest*) fr == NULL) caml_failwith("famCancelMonitor: wrong famRequest");

    if (FAMCancelMonitor( (FAMConnection*) fc, (FAMRequest*) fr ) < 0)
    {
        caml_failwith("famCancelMonitor");
    }
    return Val_unit;
}

static const enum FAMCodes famCodes_table[] = {
    FAMChanged,
    FAMDeleted,
    FAMStartExecuting,
    FAMStopExecuting,
    FAMCreated,
    FAMMoved,
    FAMAcknowledge,
    FAMExists,
    FAMEndExist
};

CAMLprim value
famcode_to_string( value ml_famCode )
{
    enum FAMCodes famCode;
    famCode = famCodes_table[Int_val(ml_famCode)];

    switch (famCode) {
        case FAMChanged:        return caml_copy_string("FAMChanged");        break;
        case FAMDeleted:        return caml_copy_string("FAMDeleted");        break;
        case FAMStartExecuting: return caml_copy_string("FAMStartExecuting"); break;
        case FAMStopExecuting:  return caml_copy_string("FAMStopExecuting");  break;
        case FAMCreated:        return caml_copy_string("FAMCreated");        break;
        case FAMMoved:          return caml_copy_string("FAMMoved");          break;
        case FAMAcknowledge:    return caml_copy_string("FAMAcknowledge");    break;
        case FAMExists:         return caml_copy_string("FAMExists");         break;
        case FAMEndExist:       return caml_copy_string("FAMEndExist");       break;
        default: caml_failwith("string_of_famCode: enum error");
    }
    return Val_unit;
}

CAMLprim value
ml_fampending( value fc )
{
    if ((FAMConnection*) fc == NULL) caml_failwith("famPending: wrong famConnection");

    return Val_int(FAMPending( (FAMConnection*) fc));
}

CAMLprim value
ml_famnextevent( value fc, value fe )
{
    if ((FAMConnection*) fc == NULL) caml_failwith("famNextEvent: wrong famConnection");
    if ((FAMEvent*) fe == NULL) caml_failwith("famNextEvent: wrong famEvent");

    if (FAMNextEvent( (FAMConnection*) fc, (FAMEvent*) fe) < 0)
    {
        caml_failwith(FamErrlist[FAMErrno]);
    }
}

CAMLprim value
famevent_code( value fe )
{
    if ((FAMEvent*) fe == NULL) caml_failwith("famEvent_code: wrong famEvent");

    switch (((FAMEvent*)fe)->code)
    {
        case FAMChanged:        return Val_int(0); break;
        case FAMDeleted:        return Val_int(1); break;
        case FAMStartExecuting: return Val_int(2); break;
        case FAMStopExecuting:  return Val_int(3); break;
        case FAMCreated:        return Val_int(4); break;
        case FAMMoved:          return Val_int(5); break;
        case FAMAcknowledge:    return Val_int(6); break;
        case FAMExists:         return Val_int(7); break;
        case FAMEndExist:       return Val_int(8); break;
        default: caml_failwith("famEvent_code: enum error");
    }
    caml_failwith("famEvent_code: enum error");
    return Val_unit;
}

CAMLprim value
famevent_filename( value fe )
{
    if ((FAMEvent*) fe == NULL) caml_failwith("famEvent_filename: wrong famEvent");

    return caml_copy_string( ((FAMEvent*)fe)->filename );
}

CAMLprim value
famevent_hostname( value fe )
{
    char *host;

    if ((FAMEvent*) fe == NULL) caml_failwith("famEvent_hostname: wrong famEvent");

    host = ((FAMEvent*)fe)->hostname;
    if (host == NULL) return caml_copy_string("");

    return caml_copy_string(host);
}

CAMLprim value
famevent_conn( value fe )
{
    if ((FAMEvent*) fe == NULL) caml_failwith("famEvent_connection: wrong famEvent");

    return (value) (((FAMEvent*)fe)->fc);
}

//CAMLprim value
//famevent_req( value fe )
//{
//    FAMRequest *_fr;

//    if ((FAMEvent*) fe == NULL) caml_failwith("famEvent_request: wrong famEvent");

//    _fr = &(((FAMEvent*)fe)->fr);
//    return (value) _fr;
//}

CAMLprim value
famevent_userdata( value fe )
{
    if ((FAMEvent*) fe == NULL) caml_failwith("famEvent_userdata: wrong famEvent");

    return (value) (((FAMEvent*)fe)->userdata);
}

CAMLprim value
famrequest_reqnum( value fr )
{
    return Val_int( ((FAMRequest*)fr)->reqnum );
}

CAMLprim value
famevent_request_num( value fe )
{
    if ((FAMEvent*) fe == NULL) caml_failwith("famEvent_request: wrong famEvent");

    return Val_int( ((FAMEvent*)fe)->fr.reqnum );
}

// vim: ts=4 sts=4 sw=4 et
